Stan Math Library: stan/math/prim/scal/prob/pareto_type_2_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

pareto_type_2_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
10 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
11 #include <stan/math/prim/scal/err/check_not_nan.hpp>
12 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/fun/log1m.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <stan/math/prim/scal/meta/include_summand.hpp>
19 #include <boost/random/variate_generator.hpp>
20 #include <cmath>
21
22
23 namespace stan {
24 namespace math {
25
26 // pareto_type_2(y|lambda, alpha) [y >= 0; lambda > 0; alpha > 0]
27 template <bool propto,
28 typename T_y, typename T_loc, typename T_scale, typename T_shape>
29 typename return_type<T_y, T_loc, T_scale, T_shape>::type
30 pareto_type_2_log(const T_y& y, const T_loc& mu, const T_scale& lambda,
31 const T_shape& alpha) {
32 static const char* function("stan::math::pareto_type_2_log");
33 typedef
34 typename stan::partials_return_type<T_y, T_loc, T_scale, T_shape>::type
35 T_partials_return;
36
37 using std::log;
38 using stan::math::value_of;
39 using stan::math::check_finite;
40 using stan::math::check_greater_or_equal;
41 using stan::math::check_positive_finite;
42 using stan::math::check_nonnegative;
43 using stan::math::check_not_nan;
44 using stan::math::check_consistent_sizes;
45 using std::log;
46
47 // check if any vectors are zero length
48 if (!(stan::length(y)
49 && stan::length(mu)
50 && stan::length(lambda)
51 && stan::length(alpha)))
52 return 0.0;
53
54 // set up return value accumulator
55 T_partials_return logp(0.0);
56
57 // validate args (here done over var, which should be OK)
58 check_greater_or_equal(function, "Random variable", y, mu);
59 check_not_nan(function, "Random variable", y);
60 check_positive_finite(function, "Scale parameter", lambda);
61 check_positive_finite(function, "Shape parameter", alpha);
62 check_consistent_sizes(function,
63 "Random variable", y,
64 "Scale parameter", lambda,
65 "Shape parameter", alpha);
66
67
68 // check if no variables are involved and prop-to
69 if (!include_summand<propto, T_y, T_loc, T_scale, T_shape>::value)
70 return 0.0;
71
72 VectorView<const T_y> y_vec(y);
73 VectorView<const T_loc> mu_vec(mu);
74 VectorView<const T_scale> lambda_vec(lambda);
75 VectorView<const T_shape> alpha_vec(alpha);
76 size_t N = max_size(y, mu, lambda, alpha);
77
78 // set up template expressions wrapping scalars into vector views
79 OperandsAndPartials<T_y, T_loc, T_scale, T_shape>
80 operands_and_partials(y, mu, lambda, alpha);
81
82 VectorBuilder<include_summand<propto, T_y, T_loc, T_scale, T_shape>
83 ::value,
84 T_partials_return, T_y, T_loc, T_scale>
85 log1p_scaled_diff(N);
86 if (include_summand<propto, T_y, T_loc, T_scale, T_shape>::value) {
87 for (size_t n = 0; n < N; n++)
88 log1p_scaled_diff[n] = log1p((value_of(y_vec[n])
89 - value_of(mu_vec[n]))
90 / value_of(lambda_vec[n]));
91 }
92
93 VectorBuilder<include_summand<propto, T_scale>::value,
94 T_partials_return, T_scale> log_lambda(length(lambda));
95 if (include_summand<propto, T_scale>::value) {
96 for (size_t n = 0; n < length(lambda); n++)
97 log_lambda[n] = log(value_of(lambda_vec[n]));
98 }
99
100 VectorBuilder<include_summand<propto, T_shape>::value,
101 T_partials_return, T_shape> log_alpha(length(alpha));
102 if (include_summand<propto, T_shape>::value) {
103 for (size_t n = 0; n < length(alpha); n++)
104 log_alpha[n] = log(value_of(alpha_vec[n]));
105 }
106
107 VectorBuilder<!is_constant_struct<T_shape>::value,
108 T_partials_return, T_shape> inv_alpha(length(alpha));
109 if (!is_constant_struct<T_shape>::value) {
110 for (size_t n = 0; n < length(alpha); n++)
111 inv_alpha[n] = 1 / value_of(alpha_vec[n]);
112 }
113
114 for (size_t n = 0; n < N; n++) {
115 const T_partials_return y_dbl = value_of(y_vec[n]);
116 const T_partials_return mu_dbl = value_of(mu_vec[n]);
117 const T_partials_return lambda_dbl = value_of(lambda_vec[n]);
118 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
119 const T_partials_return sum_dbl = lambda_dbl + y_dbl - mu_dbl;
120 const T_partials_return inv_sum = 1.0 / sum_dbl;
121 const T_partials_return alpha_div_sum = alpha_dbl / sum_dbl;
122 const T_partials_return deriv_1_2 = inv_sum + alpha_div_sum;
123
124 // // log probability
125 if (include_summand<propto, T_shape>::value)
126 logp += log_alpha[n];
127 if (include_summand<propto, T_scale>::value)
128 logp -= log_lambda[n];
129 if (include_summand<propto, T_y, T_scale, T_shape>::value)
130 logp -= (alpha_dbl + 1.0) * log1p_scaled_diff[n];
131
132 // gradients
133 if (!is_constant_struct<T_y>::value)
134 operands_and_partials.d_x1[n] -= deriv_1_2;
135 if (!is_constant_struct<T_loc>::value)
136 operands_and_partials.d_x2[n] += deriv_1_2;
137 if (!is_constant_struct<T_scale>::value)
138 operands_and_partials.d_x3[n] -= alpha_div_sum * (mu_dbl - y_dbl)
139 / lambda_dbl + inv_sum;
140 if (!is_constant_struct<T_shape>::value)
141 operands_and_partials.d_x4[n] += inv_alpha[n] - log1p_scaled_diff[n];
142 }
143 return operands_and_partials.value(logp);
144 }
145
146 template <typename T_y, typename T_loc, typename T_scale, typename T_shape>
147 inline
148 typename return_type<T_y, T_loc, T_scale, T_shape>::type
149 pareto_type_2_log(const T_y& y, const T_loc& mu,
150 const T_scale& lambda, const T_shape& alpha) {
151 return pareto_type_2_log<false>(y, mu, lambda, alpha);
152 }
153 }
154 }
155 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_greater_or_equal
bool check_greater_or_equal(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is greater or equal than low.
Definition: check_greater_or_equal.hpp:84
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::log1p
fvar< T > log1p(const fvar< T > &x)
Definition: log1p.hpp:16
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::pareto_type_2_log
return_type< T_y, T_loc, T_scale, T_shape >::type pareto_type_2_log(const T_y &y, const T_loc &mu, const T_scale &lambda, const T_shape &alpha)
Definition: pareto_type_2_log.hpp:30
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.